@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Tests for Terminal methods that account for sequences in strings"""
 
 # std imports
@@ -38,18 +37,15 @@ def test_length_cjk():
     child()
 
 
-def test_length_with_zwj_is_wrong():
-    """Because of the way Zero-Width Joiner (ZWJ) is measured, blessed gets this wrong"""
-    # But for the time being, so do many terminals (~85%), so its not a huge deal..
-    # https://ucs-detect.readthedocs.io/results.html
+def test_length_with_zwj():
+    """Test that ZWJ sequences are measured correctly like wcswidth()."""
     @as_subprocess
     def child():
         term = TestTerminal()
         # RGI_Emoji_ZWJ_Sequence  ; family: woman, woman, girl, boy
         given = term.bold_red('\U0001F469\u200D\U0001F469\u200D\U0001F467\u200D\U0001F466')
-        expected = sum((2, 0, 2, 0, 2, 0, 2))
+        expected = 2
 
-        # exercise,
         assert term.length(given) == expected
     child()
 
@@ -62,7 +58,7 @@ def test_length_ansiart():
         # this 'ansi' art contributed by xzip!impure for another project,
         # unlike most CP-437 DOS ansi art, this is actually utf-8 encoded.
         fname = os.path.join(os.path.dirname(__file__), 'wall.ans')
-        with open(fname, 'r', encoding='utf-8') as ansiart:
+        with open(fname, encoding='utf-8') as ansiart:
             lines = ansiart.readlines()
         assert term.length(lines[0]) == 67  # ^[[64C^[[34m▄▓▄
         assert term.length(lines[1]) == 75
@@ -288,6 +284,9 @@ def test_sequence_length(all_terms):
         assert term.length('x\b') == 0
         assert term.strip('x\b') == ''
 
+        # characters where wcwidth returns -1
+        assert term.length('\x00\x01\x02\x03\x04\x05') == 0
+
         # XXX why are some terminals width of 9 here ??
         assert term.length('\t') in {8, 9}
         assert term.strip('\t') == ''
